package skeb

import scala.gui.Publisher

/** A class representing a single cell in the spreadsheet. All cells must be
  * referenced from within a Sheet instance. */
abstract class Cell extends Publisher {
  
  /** A reference to the containing Sheet instance. */
  val sheet: Sheet
  /** The column index in the containing Sheet for this cell. */
  val column: Int
  /** The row index in the containing Sheet for this cell. */
  val row: Int
  
  /** Prints the Cell in a human-readable form. */
  override def toString() =
    "Cell(" + column + "#" + row + ")"
  
  /** The current expression provided by the user for this cell. This must be
    * evaluated to calculate the actual value. */
  var exp: Expression = expression.Empty
  
  /** The displayable value of this cell. This is the result of evaluating the
    * exp expression. */
  def value: expression.Constant = expression.Text("N/A")
  
  /** A method called by the user-interface when the user sets a new expression
    * for this cell.
    * @param value Either the new value as a non-empty string, or None when the
    *              cell has been emptied. */
  def userSets (value: Option[String]) = {
    // Parses the expression from the string the user provided.
    if (value.isEmpty)
      exp = expression.Empty
    else
      exp = sheet.parse(value.get)
  }
  
}